/*********************************************************************
 * Desktop Station Router for Arduino UNO
 *
 * Copyright (C) 2016 Yaasan
 *
 */

#include <avr/wdt.h>
#include <SPI.h>
#include "TrackReporterS88_DS.h"
#include "DSGatewayLib.h"


#define PROTCOL		ADDR_ACC_DCC
#define MAX_S88DECODER	1
#define MAX_ACCTBL		8

DSGatewayLib Gateway;
TrackReporterS88_DS reporter(MAX_S88DECODER);


//Task Schedule
unsigned long gPreviousL1 = 0; //   250ms interval

//Flag buffer
word gFlags[MAX_S88DECODER];


word gAccTable[MAX_ACCTBL] = { 1, 2, 5, 10, 11, 12, 13, 14  };

void setup()
{
	
	Gateway.begin();
	
	/* Clear data of the S88 decoder */
	reporter.refresh();
	
	/* Open serial port */
	Serial.begin(115200);
	while (!Serial);

	Serial.println("Desktop Station Router R0.1");
	Serial.println("2016 June 29");
	Serial.println("--------------------------------------");
	
	
	/* Load from EEPROM */
	
	
	/* EHb`hbOL(8sec) */
	wdt_enable(WDTO_8S); // WDT reset
	
	
	/* Clear */
	for( int j = 0; j < MAX_S88DECODER; j++)
	{
		gFlags[j] = 0;
	}
	
	// Power on
	delay(500);
	Gateway.SetPower(true);
	
	
	gPreviousL1 = millis();
	sei();    //ENABLE INTERRUPTION
	
}

void loop()
{
	
	
	/* Reset watchdog timer. */
	wdt_reset(); 
	
	//Button monitoring interval
	if( (millis() - gPreviousL1) >= 250)
	{
		
		reporter.refresh(MAX_S88DECODER);
		
		//Send a S88 sensor reply 
		Serial.print("@S88,");
		
		word aFlags[MAX_S88DECODER];
		
		for( int j = 0; j < MAX_S88DECODER; j++)
		{
			aFlags[j] = (reporter.getByte((j << 1) + 1) << 8) + reporter.getByte(j << 1);
			
			Serial.print(aFlags[j], HEX);
			Serial.print(",");
		}
			
		Serial.println("");
		
		
		/* Check  */
		for( int j = 0; j < MAX_S88DECODER; j++)
		{
			/* Check changes */
			for( int i = 0; i < 8; i++)
			{
				if( ((aFlags[j] >> i) & 1)  != ((gFlags[j] >> i) & 1))
				{
					
					byte aIndex = (j << 3) + i;
					
					Gateway.SetTurnout( PROTCOL + gAccTable[aIndex], (aFlags[j] >> i) & 1);
					
					delay(50);
					
					//Debug Message
					Serial.print("ACC ");
					Serial.print(gAccTable[aIndex]);
					Serial.print(" -> ");
					Serial.print((aFlags[j] >> i) & 1);
					Serial.println("");
				}
			}
			
			//Save flags
			gFlags[j] = aFlags[j];
			
		}
		
		
		//Reset task
		gPreviousL1 = millis();
	}	

}
